// Import necessary modules from d3 library
import {
  csv,
  select,
  scaleOrdinal,
  scaleLinear,
  axisBottom,
} from 'd3';
// Import scatterPlot function from scatterPlot.js file
import { scatterPlot } from './scatterPlot';

// Data set URI
const csvUrl =
  'https://gist.githubusercontent.com/sagam929/e18cebb98ecf2dce1f1d178f26ff4f34/raw/e9eccb590e0156762a6222af07f91fa245d851fa/University_Ranks.csv';

// Function to parse each row of the CSV data and select only desired columns
const parseRow = (d) => {
  return {
    'Student Population': +d['Student Population'],
    'Students to Staff Ratio':
      +d['Students to Staff Ratio'],
    'Overall Score': +d['Overall Score'],
    Teaching: +d['Teaching'],
  };
};

// Function to initialize the main visualization
const initialize = async () => {
  // Load data from CSV URI
  const data = await csv(csvUrl, parseRow);

  // Define dimensions for the SVG container
  const width = window.innerWidth;
  const height = window.innerHeight;

  // Create SVG element
  const svg = select('body')
    .append('svg')
    .attr('width', width)
    .attr('height', height);

  // Initialize scatter plot
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(data)
    .xValue((d) => d['Student Population'])
    .yValue((d) => d['Students to Staff Ratio'])
    .margin({ top: 20, right: 130, bottom: 60, left: 70 })
    .radius(5);

  // Append y label to the svg
  svg
    .append('text')
    .attr('class', 'y label')
    .attr('text-anchor', 'end')
    .attr('y', 10)
    .attr('dy', '.65em')
    .attr('dx', '-5.65em')
    .attr('transform', 'rotate(-90)')
    .attr('fill', 'Blue')
    .attr('font-size', '16px')
    .attr('font-family', 'Arial, Helvetica, sans-serif')
    .text('Students to Staff Ratio');

  // Append x label to the svg
  svg
    .append('text')
    .attr('class', 'x label')
    .attr('text-anchor', 'end')
    .attr('x', width - 450)
    .attr('y', height - 10)
    .attr('fill', 'Blue')
    .attr('font-size', '16px')
    .attr('font-family', 'Arial, Helvetica, sans-serif')
    .text('Student Population');

  // Initialize x scale
  const xScale = scaleLinear()
    .domain([
      0,
      d3.max(data, (d) => d['Student Population']),
    ])
    .range([
      plot.margin().left,
      width - plot.margin().right,
    ]);

  // Initialize x axis
  const xAxis = axisBottom(xScale);

  // Append x axis to SVG
  svg
    .append('g')
    .attr('class', 'x axis')
    .attr(
      'transform',
      `translate(0, ${height - plot.margin().bottom})`,
    )
    .call(xAxis);

  // Append scatter plot to SVG
  svg.call(plot);
};

// Call the initialize function to start the visualization
initialize();
