import {
  scaleLinear,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
} from 'd3';

// Declaring scatterPlot function
export const scatterPlot = () => {
  // declaring variables used
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;

  // The main function that generates the scatter plot
  const my = (selection) => {
    // Declaring x-axis scale
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    // Declaring y-axis scale
    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    // Declaring color scale for scatter plot circles
    const color = scaleOrdinal()
      .domain(['setosa', 'versicolor', 'virginica'])
      .range(['#440154ff', '#21908dff', '#fde725ff']);

    // Loading marks with x, y, and color values from data
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      color: color(d.species),
    }));

    // Creating circles with loaded marks
    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join('circle')
      .attr('fill', (d) => d.color)
      .attr('cx', (d) => d.x)
      .attr('cy', (d) => d.y)
      .attr('r', radius);

    // Loading y-axis
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));

    // Loading x-axis
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`,
      )
      .call(axisBottom(x));
  };

  // Helper Functions to set and get values of the function when a new object is created
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
